%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2017 Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Smart Grid Operation Problems
%
%
% Test bed 1:
% Stochastic OPF based active-reactive power dispatch
%
% This file is the same that the psopt.m file of 2014 constest,
% some of the codes of 2014 competition have been modified by
% Sergio Rivera (E-Mail: srriverar@unal.edu.co)
%
% Application of Modern Heuristic Optimization Algorithms
% for Solving Optimal Power Flow Problems
% by Sebastian Wildenhues (E-Mail: sebastian.wildenhues@uni-due.de)
% 14th February 2014
%
% Incorporating basic Particle Swarm Optimization (PSO) algorithm as
% optimization engine to solve test bed declarations V1.4.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CE_EPSO_opt(fhd,ii,jj,kk,args)
% Paste and adapt the following
% parts to your implementation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global proc
global ps
% Reinitialization of local
% function evaluation counter.
proc.i_eval=0;
% Function evaluation at which
% the last update of the global
% best solution occurred.
% Refers to the internal evaluation
% using static penalty constraint
% handling method.
proc.last_improvement=1;
% Signalizing your
% to stop running.
proc.finish=0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CE Initializations
n_samples = proc.pop_size;
ro = 0.1;
sigma = 0.85;
beta = 0.9;
q = 5;
indexRo = floor( ( 1 - ro ) * n_samples );
% indexRo = n_samples - 10;
H = zeros( 1, n_samples );
pos = zeros( n_samples, ps.D );
gammaIter = zeros( 1, 1e6 );
x_min = zeros( 1, ps.D );
x_max = zeros( 1, ps.D );
n_trials = ones( 1, ps.D );
meanPar = zeros( 1, ps.D );
stdPar = zeros( 1, ps.D );
for i = 1 : ps.D
    if i <= ps.D_cont % Continuous variables
        x_min( 1, i ) = ps.x_min( 1, i );
        x_max( 1, i ) = ps.x_max( 1, i );
        meanPar( 1, i ) = x_min( 1, i ) + rand() * ( x_max( 1, i ) - x_min( 1, i ) );
        stdPar( 1, i ) = ( x_max( 1, i ) - x_min( 1, i ) ) * 5;
    elseif i <= ps.D_cont + ps.n_OLTC % OLTC
        x_min( 1, i ) = ps.x_min( 1, i ) - 0.499;
        x_max( 1, i ) = ps.x_max( 1, i ) + 0.499;
        meanPar( 1, i ) = x_min( 1, i ) + rand() * ( x_max( 1, i ) - x_min( 1, i ) );
        stdPar( 1, i ) = ( x_max( 1, i ) - x_min( 1, i ) ) * 5;
    else % Shunt compensators
        x_min( 1, i ) = 0;
        x_max( 1, i ) = 1;
        meanPar( 1, i ) = rand();
    end
end
iter = 0;
memMeanPar( iter + 1, : ) = meanPar;
memStdPar( iter + 1, : ) = stdPar;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EPSO Initializations
global epso_par;
epso_par.pop_size = proc.pop_size;
epso_par.mutation_rate = 0.8;
epso_par.communication_probability = 0.6;
% Weights matrix
% 1 - inertia
% 2 - memory
% 3 - cooperation
% 4 - perturbation
weights = rand( epso_par.pop_size, 4 );
epso_par.init = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fitness Function Initializations
global ff
ff.coeff = ones( 1, 4 );
ff.weights = zeros( n_samples, 4 );
ff.i_sample = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%
while 1
    %%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Run CE
    while proc.i_eval <= 25000
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Generate population
        for i = 1 : ps.D
            if i <= ps.D_cont + ps.n_OLTC % Continuous variables + OLTC
                tmp_stat = normrnd( meanPar( i ), stdPar( i ), 1, n_samples );
                tmp_stat( tmp_stat < x_min( 1, i ) ) = x_min( 1, i );
                tmp_stat( tmp_stat > x_max( 1, i ) ) = x_max( 1, i );
                tmp_eval = tmp_stat;
            else % Shunt compensators
                tmp_stat = binornd( n_trials( i ), meanPar( i ), 1, n_samples );
                tmp_eval = tmp_stat + x_min( 1, i );
            end
            pos( :, i ) = tmp_eval;
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Evaluate population
        [ fit,~,~,pos,~ ] = feval( fhd,ii,jj,kk,args,pos );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Performs CE correction
        
        % Sorts states according to fitness
        [ Saux, Iaux ] = sort( fit, 2, 'descend' );
        H = zeros( 1, n_samples );
        for i = 1 : n_samples
            if i <= indexRo
                H( Iaux( i ) ) = 0;
            else
                H( Iaux( i ) ) = 1;
            end
        end
        
        % Computes gammaIter
        gamma = Saux( 1, indexRo );
        gammaIter( 1, iter + 1 ) = gamma;
        
        % Applies CE correction
        den = sum( H );
        for i = 1 : ps.D
            num = H * pos( :, i );
            tmpMean = 0;
            if den > 0
                tmpMean = ( num / den ) * ( 1 / n_trials( 1, i ) );
            end
            num = H * ( ( pos( :, i ) - tmpMean ) .* ( pos( :, i ) - tmpMean ) );
            tmpStd = 0;
            if den > 0
                tmpStd = sqrt( num / den ) * ( 1 / n_trials( 1, i ) );
            end
            meanPar( i ) = tmpMean * sigma + meanPar( i ) * ( 1 - sigma );
            
            % Update smoothing parameters
            sigmaStd = beta - beta * ( 1 - 1 / ( iter + 1 ) )^q;
            stdPar( i ) = tmpStd * sigmaStd + stdPar( i ) * ( 1 - sigmaStd );
            
            % Update sampling distribution parameters
            if i <= ps.n_gen_PG
                meanPar( i ) = round( meanPar( i ), 2 );
            elseif i <= ps.D_cont
                meanPar( i ) = round( meanPar( i ), 4 );
            elseif i <= ps.D_cont + ps.n_OLTC
                % meanPar( i ) = round( meanPar( i ), 2 );
            else
                if meanPar( i ) < 0.01
                    meanPar( i ) = 0.01;
                elseif meanPar( i ) > 0.99
                    meanPar( i ) = 0.99;
                end
            end
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Update the weights of the fitness function
        tmpMat = ff.weights( 1:ff.i_sample, : );
        tmp = max( tmpMat );
        ff.coeff = 10 .^ round( log10( max( tmp )./ tmp )  );
        ff.coeff( isinf( ff.coeff ) ) = 1;
        ff.i_sample = 0;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Update CE iteration
        iter = iter + 1;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        memMeanPar( iter + 1, : ) = meanPar;
        memStdPar( iter + 1, : ) = stdPar;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Run EPSO
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %  Initialize EPSO population
    if epso_par.init == 1
        Xmin = x_min;
        Xmax = x_max;
        Vmin = -Xmax + Xmin;
        Vmax = -Vmin;
        vel = zeros( epso_par.pop_size, ps.D );
        for i = 1 : epso_par.pop_size
              vel( i, : ) = ( Vmin + ( Vmax - Vmin ) .* rand( 1, ps.D ) );
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Evaluate population
        [ fit,~,~,pos,~ ] = feval( fhd,ii,jj,kk,args,pos );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Update global best
        [ gbestval, gbestid ] = min( fit );
        gbest = pos( gbestid, : );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % UPDATE individual best
        % Individual best position ever of the particles of current population
        myBestPos = pos;
        % Fitness of the individual best position ever of the particles of current population
        myBestPosFit = fit;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        epso_par.init = 0;
        
        gammaIter = gammaIter( 1, 1:iter );
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % COPY CURRENT population
    copy_pos = pos;
    copy_vel = vel;
    copy_weights = weights;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Apply EPSO movement rule
    for i = 1 : epso_par.pop_size
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Mutate weights of the particles of the COPIED population
        copy_weights( i, : ) = MUTATE_WEIGHTS( weights( i, : ), epso_par.mutation_rate );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new velocity for the particles of the COPIED population
        copy_vel( i, : ) = EPSO_COMPUTE_NEW_VEL( ps.D, copy_pos( i, : ), myBestPos( i, : ), gbest, ...
            copy_vel( i, : ), Vmin, Vmax, copy_weights( i, : ), epso_par.communication_probability );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new position for the particles of the COPIED population
        [ copy_pos( i, : ), copy_vel( i, : ) ] = COMPUTE_NEW_POS( copy_pos( i, : ), copy_vel( i, : ) );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
               
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new velocity for the particles of the CURRENT population
        vel( i, : ) = EPSO_COMPUTE_NEW_VEL( ps.D, pos( i, : ), myBestPos( i, : ), gbest, ...
            vel( i, : ), Vmin, Vmax, weights( i, : ), epso_par.communication_probability );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new position for the particles of the CURRENT population
        [ pos( i, : ), vel( i, : ) ] = COMPUTE_NEW_POS( pos( i, : ), vel( i, : ) );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Enforce search space limits of the COPIED population
    [ copy_pos, copy_vel ] = ENFORCE_POS_LIMITS( ps.D, epso_par.pop_size, copy_pos, Xmin, Xmax, copy_vel, Vmin, Vmax );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Enforce search space limits of the CURRENT population
    [ pos, vel ] = ENFORCE_POS_LIMITS( ps.D, epso_par.pop_size, pos, Xmin, Xmax, vel, Vmin, Vmax );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate the COPIED population
    [ copy_fit,~,~,copy_pos,~ ] = feval( fhd,ii,jj,kk,args,copy_pos );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate the CURRENT population
    [ fit,~,~,pos,~ ] = feval( fhd,ii,jj,kk,args,pos );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % CREATE NEW population to replace CURRENT population
    selParNewSwarm = ( copy_fit < fit );
    for i = 1 : epso_par.pop_size
        if selParNewSwarm( i )
            fit( i ) = copy_fit( i );
            pos( i, : ) = copy_pos( i, : );
            vel( i, : ) = copy_vel( i, : );
            weights( i, : ) = copy_weights( i, : );
        end
        if fit( i ) < myBestPosFit( i )
            myBestPos( i, : ) = pos( i, : );
            myBestPosFit( i ) = fit( i );
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % UPDATE GLOBAL BEST
    [ tmpgbestval, gbestid ] = min( fit );
    if tmpgbestval < gbestval
        gbestval = tmpgbestval;
        gbest = pos( gbestid, : );
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%
    if proc.finish       
        return;
    end
    %%%%%%%%%%%%%%
    
    % To prematurely stop
    % running the current trial,
    % you may use a statement
    % as the following.
    % However, note that
    % storage of intermediate
    % results must be done
    % manually since no ASCII
    % file output will be
    % provided in that case.
    % As an example, retrieve
    % relevant information
    % from cell array res.
    %         %%%%%%%%%%%%%%%%%%%%
    %         if proc.i_eval>=1000
    %             return;
    %         end
    %         %%%%%%%%%%%%%%%%%%%%
end