%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2017 Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Smart Grid Operation Problems
% 
% 
% Test bed 1:
% Stochastic OPF based active-reactive power dispatch 
%
%
% Sergio Rivera (E-Mail: srriverar@unal.edu.co)
%
% For cases 3 and 6, it is considered that the 1 renewable generator is a wind 
% generator at bus 2 and that at buses 6 and 9 there are two generators, a solar 
% generator and a small-hydro generator. It is well known that in several parts 
% of the world the solar irradiance probability distribution follows a lognormal 
% distribution and the river flow follows a gumbel distribution. Additionally, 
% there is a relation between the solar irradiance and the available real power; 
% and between the river flow and the available real power. In this way, it is 
% possible to get the probability distribution of the available real power. The 
% file named SolarWindHydroStochastic.m has the mentioned process in order to get
% the 5000 montecarlo scenarios for the available real power.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                  
                    %% WIND PARAMETERS
                    %% Inicializacion de Constantes Eolico

                    Vi=5;           % velocidad de viento arranque (m/s)
                    Vr=15;          % Velocidad de viento nominal  (m/s)
                    Vo=45;          % Velocidad de viento parada   (m/s)
                    Wr=150;         % Potencia nominal aerogenador (Mw)
                    a= 15;          % Coeficiente lineal aerogenrador
                    b= -75;         % Coeficiente lineal.
                    sg= 15.9577;    % Parametro de escala distribucin Rayleigh.
                    c=sqrt(2)*sg;   % factor de escala Weilbull
                    k= 2;           % Factor de forma Weilbull
                   
                    %% SOLAR PARAMETERS
                    %% Inicializacion de Constantes Solar

                    Psr= 100; % [MW] Equivalent rated power output of the PV generator
                    Gstd= 775; %[W/m2] Solar irradiation in the standard enviroment
                    Rc=116; % [W/m2] A certain irradiation 
                    Wmax= 150;% 
                    lambda= 6; % 
                    beta= 0.25;% desviacin estandard

                    %% SMALL-HYDRO PARAMETERS
                    %% Inicializacion de Constantes Hydro
                    
                    p=1000;     %p=densidad agua
                    nt=0.9;     %nt=eficiencia_turbina
                    ng=0.95;    %ng=eficiencia_generador
                    nm=0.98;    %nm=eficiencia_acople_turbina_generador
                    h=20;       %h=altura_presa
                    mu=15.23;   %mu=valor promedio
                    sig=1.15;   %sig=desviacion media cuadratica
                                       
                    kk=9.81*p*nt*ng*nm*h;
 
%% MONTECARLO SCENARIOS GENERATION

N=5000;         % Numero de pruebas
         
for hi=1:N
                        v2(hi)=wblrnd(c,k); %% generacion viento 2
                        
                        %v4(hi)=wblrnd(c,k); %% generacion viento 4
                        G(hi)=lognrnd(lambda,beta); %% generacion solar 4
                        Q(hi)=evrnd(mu,sig);
                        
                        %v6(hi)=wblrnd(c,k); %% generacion viento 6
                        G2(hi)=lognrnd(lambda,beta);
                        Q2(hi)=evrnd(mu,sig);

                        if (v2(hi)<Vi)||(v2(hi)>Vo)%% calculo de potencia eolica 2
                            W2(hi)=0;
                        elseif v2(hi)<Vr && Vi<v2(hi)
                            W2(hi)=a*v2(hi)+b;
                        else
                            W2(hi)=Wr;
                        end

 
                        if ((G(hi)<Rc)&&(G(hi)>0))%% calculo de potencia eolica
                                W4(hi)=Psr*(G(hi)^2/(Gstd*Rc));%Hay una corte W(i) maximo dado cuando G(i)=Rc
                        elseif G(hi)>Rc
                                W4(hi)=Psr*(G(hi)/Gstd);
                        end
                        
                        W4(hi)=W4(hi)+((Q(hi)*kk)/1e6);

                        if ((G2(hi)<Rc)&&(G2(hi)>0))%% calculo de potencia eolica
                                W6(hi)=Psr*(G2(hi)^2/(Gstd*Rc));%Hay una corte W(i) maximo dado cuando G(i)=Rc
                        elseif G(hi)>Rc
                                W6(hi)=Psr*(G2(hi)/Gstd);
                        end
                        
                        W6(hi)=W6(hi)+((Q2(hi)*kk)/1e6);

                        
end

   global ScenarioType
    
   ScenarioType=6;

   figure(1)
   subplot(2,1,1)
   hist(v2)
   xlabel('wind speed (m/s) for wind generator at bus 2')
   ylabel('frequency')
   subplot(2,1,2)
   hist(W2)
   xlabel('available real power (MW) for wind generator at bus 2')
   ylabel('frequency')
 
   figure(2)
   subplot(3,1,1)
   hist(G)
   xlabel('solar irradiance (W/m^2) for solar generator at bus 6')
   ylabel('frequency')
   subplot(3,1,2)
   hist(Q)
   xlabel('river flow (m^3/s) for small-hydro generator at bus 6')
   ylabel('frequency')
   subplot(3,1,3)
   hist(W4)
   xlabel('available real power (MW) for solar and small-hydro generators at bus 6')
   ylabel('frequency')
   
   figure(3)
   subplot(3,1,1)
   hist(G2)
   xlabel('solar irradiance (W/m^2) for solar generator at bus 9')
   ylabel('frequency')
   subplot(3,1,2)
   hist(Q2)
   xlabel('river flow (m^3/s) for small-hydro generator at bus 9')
   ylabel('frequency')
   subplot(3,1,3)
   hist(W6)
   xlabel('available real power (MW) for solar and small-hydro generators at bus 9')
   ylabel('frequency')
   